@@ -27,6 +27,7 @@ import select
 import socket
 import struct
 import threading
+import traceback
 
 from Crypto.Cipher import AES
 
@@ -109,10 +110,16 @@ def _ioworker():
             workitem = ioqueue.popleft()
             # structure is function, args, list to append to ,event to set
             if isinstance(workitem[1], tuple):  # positional arguments
-                workitem[2].append(workitem[0](*workitem[1]))
+                try:
+                    workitem[2].append(workitem[0](*workitem[1]))
+                except Exception:
+                    traceback.print_exc()
                 workitem[3].set()
             elif isinstance(workitem[1], dict):
-                workitem[2].append(workitem[0](**workitem[1]))
+                try:
+                    workitem[2].append(workitem[0](**workitem[1]))
+                except Exception:
+                    traceback.print_exc()
                 workitem[3].set()
             elif workitem[0] == 'wait':
                 if len(rdylist) > 0:
@@ -139,7 +146,10 @@ def _io_apply(function, args):
 def _io_sendto(mysocket, packet, sockaddr):
     #Want sendto to act reasonably sane..
     mysocket.setblocking(1)
-    mysocket.sendto(packet, sockaddr)
+    try:
+        mysocket.sendto(packet, sockaddr)
+    except Exception:
+        pass
 
 
 def _io_recvfrom(mysocket, size):
@@ -345,6 +355,7 @@ class Session(object):
         self.privlevel = 4
         self.maxtimeout = 3  # be aggressive about giving up on initial packet
         self.incommand = False
+        self.nameonly = 16  # default to name only lookups in RAKP exchange
         self.initialized = True
         self.cleaningup = False
         self.lastpayload = None
@@ -1159,7 +1170,7 @@ class Session(object):
             list(struct.unpack("4B",
                  struct.pack("<I", self.pendingsessionid))) +\
             list(struct.unpack("16B", self.randombytes)) +\
-            [self.privlevel, 0, 0] +\
+            [self.nameonly | self.privlevel, 0, 0] +\
             [userlen] +\
             list(struct.unpack("%dB" % userlen, self.userid))
         self.sessioncontext = "EXPECTINGRAKP2"
@@ -1197,7 +1208,7 @@ class Session(object):
         userlen = len(self.userid)
         hmacdata = struct.pack("<II", localsid, self.pendingsessionid) +\
             self.randombytes + self.remoterandombytes + self.remoteguid +\
-            struct.pack("2B", self.privlevel, userlen) +\
+            struct.pack("2B", self.nameonly | self.privlevel, userlen) +\
             self.userid
         expectedhash = hmac.new(self.password, hmacdata, hashlib.sha1).digest()
         givenhash = struct.pack("%dB" % len(data[40:]), *data[40:])
@@ -1209,7 +1220,8 @@ class Session(object):
         # to store the keys
         self.sik = hmac.new(self.kg,
                             self.randombytes + self.remoterandombytes +
-                            struct.pack("2B", self.privlevel, userlen) +
+                            struct.pack("2B", self.nameonly | self.privlevel,
+                                        userlen) +
                             self.userid, hashlib.sha1).digest()
         self.k1 = hmac.new(self.sik, '\x01' * 20, hashlib.sha1).digest()
         self.k2 = hmac.new(self.sik, '\x02' * 20, hashlib.sha1).digest()
@@ -1225,7 +1237,8 @@ class Session(object):
             list(struct.unpack("4B", struct.pack("<I", self.pendingsessionid)))
         hmacdata = self.remoterandombytes +\
             struct.pack("<I", self.localsid) +\
-            struct.pack("2B", self.privlevel, len(self.userid)) +\
+            struct.pack("2B", self.nameonly | self.privlevel,
+                        len(self.userid)) +\
             self.userid
         authcode = hmac.new(self.password, hmacdata, hashlib.sha1).digest()
         payload += list(struct.unpack("%dB" % len(authcode), authcode))
